from datetime import timedelta

from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.db.models import Q
from django.http import HttpRequest
from django.urls import reverse
from django.utils import timezone
from .app_settings import app_settings
from .forms import UserSwitchForm
from .models import SimulatedUserSession


UserModel = get_user_model()


# noinspection PyProtectedMember
def mimicry_context(request: HttpRequest) -> dict[str, any]:
    """
    Context Processor: mimicry_context

    Provides context variables related to the user simulation feature in the app.

    Parameters:
    - request (HttpRequest): The current HttpRequest object.

    Returns:
    - dict: A dictionary containing the necessary context variables.

    Context Variables:
    - simulate_user_form (UserSwitchForm): The form used to switch or simulate different users.
    - switch_simulated_user_url (str): The URL endpoint for switching simulated users.
    - show_mimicry_control_bar (bool): Indicates if the simulation control bar should be shown.
    - mimicry_control_bar_enabled (bool): Indicates if the control bar is active/enabled.
    - time_since_last_change: Indicates the time passed since the last simulation change.

    Functionality:
    - Determines the current state of the user, whether they are in a simulation or not.
    - Checks conditions and settings to decide the visibility and activity state of the simulation control bar.
    - Sets the initial data for the UserSwitchForm based on the current simulation state.

    Usage:
    This context processor should be added to the 'context_processors' option in Django's template settings to
    ensure that the required context variables are available in every template.
    """
    control_condition: bool = getattr(request.real_user, app_settings.MIMICRY_FEATURE_CONTROL_CONDITION, False)
    primary_key_name: str = UserModel._meta.pk.name
    if isinstance(request.user, AnonymousUser):
        simulated_user_pk: str = 'unauthenticated'
    elif request.user != request.real_user:
        simulated_user_pk: str | int = getattr(request.user, primary_key_name)
    else:
        simulated_user_pk: None = None
    if control_condition:
        hide_bar: bool = SimulatedUserSession.objects.filter(
            Q(real_session_key=request.session.session_key) |
            Q(simulated_session_key=request.session.session_key),
            hide_bar=True
        ).exists()
    else:
        hide_bar: bool = True
    try:
        simulated_session: SimulatedUserSession = SimulatedUserSession.objects.get(
            simulated_session_key=request.session.session_key
        )
        time_since_last_change: timedelta = timezone.now() - simulated_session.created_at
    except SimulatedUserSession.DoesNotExist:
        time_since_last_change: str = "Not in a simulation"

    form_initial_data: dict[str, any] = {
        'user_pk': simulated_user_pk if simulated_user_pk != 'unauthenticated' else None,
        'username': '',
        'email': '',
        'unauthenticated': simulated_user_pk == 'unauthenticated',
        'enabled': simulated_user_pk is not None,
        'hide': hide_bar
    }
    form: UserSwitchForm = UserSwitchForm(initial=form_initial_data)

    if control_condition:
        show_bar: bool = not hide_bar
    else:
        show_bar: bool = False

    if not show_bar:
        bar_enabled: bool = False
    else:
        bar_enabled: bool = simulated_user_pk is not None

    if not app_settings.ENABLE_MIMICRY:
        bar_enabled: bool = False

    context: dict[str, any] = {
        'mimicry_form': form,
        'switch_simulated_user_url': reverse('mimicry_switch_user'),
        'show_mimicry_control_bar': show_bar,
        'mimicry_control_bar_enabled': bar_enabled,
        'time_since_last_change': time_since_last_change
    }

    return context



from django.http import HttpResponseForbidden, HttpRequest, HttpResponse
from django.urls import reverse
from .app_settings import app_settings
from .utils import coordinate_request_with_simulated_session


class MimicryMiddleware:
    """
    Middleware to facilitate the user simulation feature.

    This middleware performs two main functions:
    1. Coordinates the request with the simulated user session, ensuring that
       `request.user` is set to the simulated user and `request.real_user` is set to the actual user.
    2. If ONLY_ALLOW_SIMULATED_GET_AND_HEAD_REQUESTS setting is enabled, ensures that
       only GET and HEAD requests are allowed during user simulation except for the 'mimicry_switch_user' endpoint.

    Attributes:
    - get_response: A callable to get the response using the provided request.

    Methods:
    - __call__: Method that gets executed for each request before reaching the view.
    """

    def __init__(self, get_response) -> None:
        """
        Initializes the middleware.

        Args:
        - get_response: A callable which takes a request and returns a response.
                        It could be the next middleware in line or the final view.
        """
        self.get_response = get_response

    # noinspection PyProtectedMember
    def __call__(self, request: HttpRequest) -> HttpResponseForbidden | HttpResponse:
        """
        Method that gets executed for each request before it reaches the view.

        Args:
        - request: The current request instance.

        Returns:
        - A HttpResponse object.
        """

        # Update the request with the simulated user's session details.
        request: HttpRequest = coordinate_request_with_simulated_session(request)

        # Check if ONLY_ALLOW_SIMULATED_GET_REQUESTS setting is enabled
        # and block non-GET requests during user simulation.
        # The 'simulate_user_switch_user' endpoint is exempted.
        if app_settings.ONLY_ALLOW_SIMULATED_GET_AND_HEAD_REQUESTS:
            if (
                request.method not in ["GET", "HEAD"] and
                request.real_user != request.user and
                not request.path.startswith(reverse('mimicry_switch_user'))
            ):
                return HttpResponseForbidden("Only GET and HEAD requests allowed in simulation mode")

        # Continue processing and get the response
        response: HttpResponse = self.get_response(request)
        return response



"""
Mimicry
------------

The Mimicry app provides functionality for simulating user sessions
and actions, allowing site administrators to view the website as if
they are a specific user, aiding in debugging and user experience enhancements.

This module sets the default application configuration class.
"""

# Point to the default AppConfig class for the 'mimicry' app.
default_app_config = 'mimicry.apps.MimicryConfig'



from datetime import datetime
from functools import wraps
from django.contrib.auth import get_user_model, login
from django.contrib.auth.models import AnonymousUser
from django.core.exceptions import PermissionDenied
from django.db import IntegrityError
from django.http import HttpRequest
from django.utils import timezone
from .app_settings import app_settings
from .models import SimulatedUserSession, SimulatedUserAction


UserModel = get_user_model()


def reset_real_user(view_func: any = None, *, request: HttpRequest | None = None):
    """
    Decorator function to reset the 'real_user' attribute on the request object.
    If used without arguments, it will also reset 'simulated_user_pk' if it exists.

    Args:
    - view_func: View function to be wrapped.
    - request: The request object.

    Returns:
    - Function: The wrapped function or the decorator itself.
    """
    # noinspection PyShadowingNames
    def decorator(view_func):
        # noinspection PyShadowingNames
        @wraps(view_func)
        def _wrapped_view(request, *args, **kwargs):
            request.real_user = request.user
            if getattr(request,  'simulated_user_pk', None):
                request.simulated_user_pk = None
            return view_func(request, *args, **kwargs)
        return _wrapped_view

    if view_func is not None:
        return decorator(view_func)

    if request is not None:
        request.real_user = request.user
        if getattr(request, 'simulated_user_pk', None):
            request.simulated_user_pk = None


# noinspection PyProtectedMember
def new_simulated_session(request: HttpRequest, hide_bar: bool = False) -> HttpRequest:
    """
    Create a new simulated session for the user.

    Args:
    - request: The request object.
    - hide_bar: Boolean indicating if the simulation bar should be hidden.

    Returns:
    - HttpRequest: Modified request object.
    """
    control_condition: bool = getattr(request.real_user, app_settings.MIMICRY_FEATURE_CONTROL_CONDITION, False)
    request.simulated_user_pk = getattr(request, 'simulated_user_pk', None)
    if control_condition and request.simulated_user_pk:
        real_session_key: str | None = request.session.session_key
        if not (request.simulated_user_pk == 'unauthenticated' or request.simulated_user_pk is None):
            request.session['_auth_user_id'] = request.simulated_user_pk

        simulated_session: SimulatedUserSession = SimulatedUserSession()
        simulated_session.real_user = request.real_user
        if hide_bar:
            request: HttpRequest = change_user(request, request.real_user)
            simulated_session.hide_bar = True
            simulated_session.closed_at = timezone.now()
        else:
            if request.simulated_user_pk == 'unauthenticated':
                request: HttpRequest = change_user(request, AnonymousUser())
            else:
                try:
                    primary_key_field: str = UserModel._meta.pk.name
                    filter_kwargs: dict[str, str] = {primary_key_field: request.simulated_user_pk}
                    user_to_login: UserModel | AnonymousUser = UserModel.objects.get(**filter_kwargs)
                    request: HttpRequest = change_user(request, user_to_login)
                except UserModel.DoesNotExist:
                    request: HttpRequest = change_user(request, AnonymousUser())
            if isinstance(request.user, AnonymousUser):
                simulated_session.simulated_user = None
            else:
                simulated_session.simulated_user = request.user
        simulated_session.real_session_key = real_session_key
        simulated_session.simulated_session_key = request.session.session_key
        try:
            simulated_session.save()
        except IntegrityError:
            pass
        request.session['_is_simulated_user_session'] = True

    return request


def set_hide_bar(request: HttpRequest, hide_bar_status: bool = True) -> HttpRequest:
    """
    Sets the 'hide_bar' status for the current simulated session.

    Args:
    - request: The request object.
    - hide_bar_status: Boolean indicating the desired 'hide_bar' status.

    Returns:
    - HttpRequest: Modified request object.
    """
    control_condition: bool = getattr(request.real_user, app_settings.MIMICRY_FEATURE_CONTROL_CONDITION, False)
    if control_condition:
        try:
            simulated_session: SimulatedUserSession = SimulatedUserSession.objects.get(
                simulated_session_key=request.session.session_key, closed_at__isnull=True
            )
            simulated_session.closed_at = timezone.now()
            simulated_session.hide_bar = hide_bar_status
            simulated_session.save()
        except SimulatedUserSession.DoesNotExist:
            request.simulated_user_pk = 'unauthenticated'
            request: HttpRequest = new_simulated_session(request, hide_bar=True)
    return request


def disable_bar(request: HttpRequest) -> HttpRequest:
    """
    Disables the simulation bar and ends the current simulated session.

    Args:
    - request: The request object.

    Returns:
    - HttpRequest: Modified request object.
    """
    control_condition: bool = getattr(request.real_user, app_settings.MIMICRY_FEATURE_CONTROL_CONDITION, False)
    if control_condition:
        try:
            simulated_session: SimulatedUserSession = SimulatedUserSession.objects.get(
                simulated_session_key=request.session.session_key, closed_at__isnull=True
            )
            simulated_session.closed_at = timezone.now()
            simulated_session.save()
            request.real_user = simulated_session.real_user
            request: HttpRequest = change_user(request, request.real_user)
            request.simulation_created_at = None
        except SimulatedUserSession.DoesNotExist:
            pass
    return request


def coordinate_request_with_simulated_session(request: HttpRequest) -> HttpRequest:
    """
    Coordinate the request with the currently active simulated session.

    Args:
    - request: The request object.

    Returns:
    - HttpRequest: Modified request object.
    """
    request.real_user = getattr(request, 'real_user', request.user)
    _is_simulated_user_session: bool = getattr(request.session, '_is_simulated_user_session', False)
    if app_settings.ENABLE_MIMICRY:
        session_key: str = request.session.session_key
        if _is_simulated_user_session:
            simulated_session: SimulatedUserSession = SimulatedUserSession.objects.filter(
                simulated_session_key=session_key
            ).first()
            if simulated_session and simulated_session.closed_at:
                request.real_user = simulated_session.real_user
                simulated_session: None = None

            if simulated_session:
                simulated_session.save()
                active_simulated_session: SimulatedUserSession = SimulatedUserSession.objects.filter(
                    simulated_session_key=session_key, closed_at__isnull=True
                ).first()
                if active_simulated_session and not simulated_session.hide_bar:
                    request.real_user = active_simulated_session.real_user
                    if active_simulated_session.simulated_user is None:
                        request: HttpRequest = change_user(request, AnonymousUser())
                        log_simulated_action(
                            request,
                            active_simulated_session,
                            f"{request.real_user.username} simulating AnonymousUser accessed {request.path}"
                        )
                    else:
                        request: HttpRequest = change_user(request, active_simulated_session.simulated_user)
                        log_simulated_action(
                            request,
                            active_simulated_session,
                            f"{request.real_user.username} simulating user: {request.user.username} accessed"
                            f" {request.path}"
                        )
                    request.simulation_created_at = active_simulated_session.created_at
                else:
                    request.real_user = simulated_session.real_user
                    request: HttpRequest = change_user(request, request.real_user)
                    request.simulation_created_at = None
            else:
                request: HttpRequest = change_user(request, request.real_user)
                request.simulation_created_at = None
        else:
            request: HttpRequest = change_user(request, request.real_user)
            request.simulation_created_at = None
    else:
        request: HttpRequest = change_user(request, request.real_user)
        request.simulation_created_at = None

    return request


def change_user(request: HttpRequest, user: UserModel | AnonymousUser) -> HttpRequest:
    """
    Change the user of the current request.

    Args:
    - request: The request object.
    - user: The user object or AnonymousUser to be set on the request.

    Returns:
    - HttpRequest: Modified request object.
    """
    if request.user != user:
        real_user: UserModel | AnonymousUser = getattr(request, 'real_user', request.user)
        simulation_created_at: datetime | None = getattr(request, 'simulation_created_at', None)
        simulated_user_pk: str | None = getattr(request, 'simulated_user_pk', None)
        if isinstance(user, UserModel):
            if check_user_simulation_permissions(request.real_user, user):
                user.backend = app_settings.MIMICRY_AUTHENTICATION_BACKEND
                login(request, user)
                request.session.save()
            else:
                raise PermissionDenied("You don't have permission to access this user.")
        else:
            if check_user_simulation_permissions(request.real_user, AnonymousUser()):
                request.session.clear()
                request.session.create()
                request.user = AnonymousUser()
            else:
                raise PermissionDenied("You don't have permission to access this user.")
        request.real_user = real_user
        request.simulation_created_at = simulation_created_at
        request.simulated_user_pk = simulated_user_pk

    return request


def check_user_simulation_permissions(real_user: UserModel, simulated_user: UserModel) -> bool:
    if isinstance(real_user, UserModel):
        if isinstance(simulated_user, UserModel) or isinstance(simulated_user, AnonymousUser):
            if isinstance(simulated_user, AnonymousUser):
                return True
            else:
                permission_settings: list[dict[str, str]] = app_settings.MIMICRY_PERMISSIONS
                permission_settings.append(
                    {"SIMULATED_USER_ATTRIBUTE": "_ALWAYS_TRUE", "REAL_USER_ATTRIBUTE": "is_staff"}
                )
                for setting in permission_settings:
                    simulated_user_attribute: str = setting["SIMULATED_USER_ATTRIBUTE"]
                    real_user_attribute: str = setting["REAL_USER_ATTRIBUTE"]
                    if simulated_user_attribute == "_ALWAYS_TRUE":
                        simulated_user_attribute_results: bool = True
                    else:
                        simulated_user_attribute_results = getattr(simulated_user, simulated_user_attribute, None)
                        if type(simulated_user_attribute_results) is not bool:
                            raise TypeError(f"Did not find {simulated_user_attribute} to be bool for simulated_user")
                    real_user_attribute_results = getattr(real_user, real_user_attribute, None)
                    if type(real_user_attribute_results) is not bool:
                        raise TypeError(f"Did not find {real_user_attribute_results} to be bool for real_user")
                    if simulated_user_attribute_results:
                        return real_user_attribute_results
                    else:
                        continue

        else:
            raise TypeError("simulated_user must be an instance of your user model or AnonymousUser")
    else:
        raise TypeError("real_user must be an instance of your user model")


def log_simulated_action(
        request: HttpRequest, simulated_session_instance: SimulatedUserSession, action_desc: str
) -> None:
    """
    Log an action performed during a simulated session.

    Args:
    - request: The request object.
    - simulated_session_instance: Instance of the current simulated session.
    - action_desc: Description of the action performed.

    Returns:
    - None
    """
    if request.real_user != request.user:
        try:
            simulated_user_action = SimulatedUserAction.objects.create(
                simulated_user_session=simulated_session_instance,
                action=action_desc
            )
            simulated_user_action.save()
        except SimulatedUserSession.DoesNotExist:
            pass



# Generated by Django 4.2.4 on 2023-08-11 22:17

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='SimulatedUserSession',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('real_session_key', models.CharField(max_length=40)),
                ('simulated_session_key', models.CharField(max_length=40, unique=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('closed_at', models.DateTimeField(blank=True, null=True)),
                ('hide_bar', models.BooleanField(default=False)),
                ('real_user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='real_user_sessions', to=settings.AUTH_USER_MODEL)),
                ('simulated_user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='simulated_user_sessions', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='SimulatedUserAction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('action', models.TextField()),
                ('timestamp', models.DateTimeField(auto_now_add=True)),
                ('simulated_user_session', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='simulated_session_actions', to='mimicry.simulatedusersession')),
            ],
        ),
    ]



from django.conf import settings
from django.core.mail import EmailMultiAlternatives
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.template.loader import render_to_string
from .app_settings import app_settings
from .models import SimulatedUserSession


@receiver(post_save, sender=SimulatedUserSession)
def send_simulation_notification(sender, instance, created, **kwargs) -> None:
    """
    Sends an email notification to the real user when a new simulated user session is created.

    This function is connected to the post_save signal of the SimulatedUserSession model.
    It is triggered every time an instance of this model is saved. The function checks
    if this is a new record and, if so, prepares and sends an email notification to the real user.

    Parameters:
    - sender: The model class. (SimulatedUserSession in this case)
    - instance: The actual instance being saved.
    - created: Boolean indicating whether this instance is being created.
               If it's an update to an existing instance, created is False.
    - **kwargs: Additional keyword arguments.

    Email Templates Used:
    - 'simulateuser/simulation_email_subject.txt': Used for the email subject.
    - 'simulateuser/simulation_email.txt': Used for the plain text content of the email.
    - 'simulateuser/simulation_email.html': Used for the HTML content of the email.

    The templates will be rendered with a context containing the following variables:
    - real_user: The actual user who initiated the simulation.
    - simulated_user: The user being simulated.
    - timestamp: The time when the simulation started.
    """

    if created and app_settings.ENABLE_SIMULATE_USER_NOTIFICATIONS:  # Check if this is a new record
        context: dict[str, any] = {
            'real_user': instance.real_user,
            'simulated_user': instance.simulated_user,
            'timestamp': instance.created_at
        }

        # Render the email subject and contents using the specified templates and context
        subject: str = render_to_string('mimicry/simulation_email_subject.txt', context).strip()
        text_content: str = render_to_string('mimicry/simulation_email.txt', context)
        html_content: str = render_to_string('mimicry/simulation_email.html', context)

        # Create a multipart email with both plain text and HTML content
        email: EmailMultiAlternatives = EmailMultiAlternatives(
            subject, text_content, settings.EMAIL_HOST_USER, [instance.real_user.email]
        )
        email.attach_alternative(html_content, "text/html")

        # Send the email
        email.send()



from datetime import timedelta, datetime
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser
from django.db import models
from django.utils import timezone
from .app_settings import app_settings


UserModel = get_user_model()


class SimulatedUserSession(models.Model):
    """
    Model representing a simulated user session.

    Attributes:
        - real_user: The actual user initiating the simulation.
        - simulated_user: The user being simulated. Can be None, representing an anonymous user.
        - real_session_key: Session key of the real user.
        - simulated_session_key: Session key of the simulated user.
        - created_at: Timestamp when the simulation session was created.
        - closed_at: Timestamp when the simulation session was closed or expired.
        - hide_bar: Flag to determine if the control bar should be hidden during the simulation.
    """

    # Fields
    real_user: UserModel = models.ForeignKey(UserModel, related_name="real_user_sessions", on_delete=models.CASCADE)
    simulated_user: UserModel | None = models.ForeignKey(
        UserModel, related_name="simulated_user_sessions", null=True, blank=True, on_delete=models.SET_NULL
    )
    real_session_key: str = models.CharField(max_length=40)
    simulated_session_key: str = models.CharField(max_length=40, unique=True)
    created_at: datetime = models.DateTimeField(auto_now_add=True)
    closed_at: datetime | None = models.DateTimeField(null=True, blank=True)
    hide_bar: bool = models.BooleanField(default=False)

    def get_simulated_user(self) -> models.ForeignKey | AnonymousUser:
        """
        Returns the simulated user. If no user is being simulated, returns an AnonymousUser instance.

        Returns:
            (models.ForeignKey | AnonymousUser): The user being simulated or an anonymous user.
        """
        if self.simulated_user:
            return self.simulated_user
        return AnonymousUser()

    @property
    def simulated_user_or_anonymous(self) -> models.ForeignKey | AnonymousUser:
        """
        Property returning the simulated user or an anonymous user.

        Returns:
            (models.ForeignKey | AnonymousUser): The user being simulated or an anonymous user.
        """
        return self.get_simulated_user()

    def save(self, *args, **kwargs) -> None:
        """
        Override the save method to:
            - Close any open simulation sessions of the same user.
            - Expire sessions that exceed the simulated session expiry time.
            - Delete sessions that exceed the simulated session retention period.

        Args:
            *args, **kwargs: Arguments passed to the base save method.
        """
        SimulatedUserSession.objects.filter(
            real_user=self.real_user, closed_at__isnull=True
        ).update(closed_at=timezone.now())

        cutoff_expiration: datetime = timezone.now() - timedelta(seconds=app_settings.SIMULATED_SESSION_EXPIRY)
        SimulatedUserSession.objects.filter(
            created_at__lt=cutoff_expiration, closed_at__isnull=True
        ).update(closed_at=timezone.now())

        cutoff_date: datetime = timezone.now() - timedelta(days=app_settings.SIMULATED_SESSION_RETENTION)
        SimulatedUserSession.objects.filter(created_at__lt=cutoff_date).delete()

        super(SimulatedUserSession, self).save(*args, **kwargs)

    def __str__(self) -> str:
        """
        String representation of the model.

        Returns:
            (str): A string representation showing who is simulating as whom and when it was created.
        """
        return f"Session by {self.real_user} as {self.simulated_user} created at {self.created_at}"


class SimulatedUserAction(models.Model):
    """
    Model representing an action performed during a simulated user session.

    Attributes:
        - simulated_user_session: The SimulatedUserSession in which the action was performed.
        - action: Text description of the action performed.
        - timestamp: Timestamp when the action was performed.
    """

    # Fields
    simulated_user_session: SimulatedUserSession = models.ForeignKey(
        SimulatedUserSession, related_name='simulated_session_actions', on_delete=models.CASCADE
    )
    action: str = models.TextField()
    timestamp: datetime = models.DateTimeField(auto_now_add=True)

    def __str__(self) -> str:
        """
        String representation of the model.

        Returns:
            (str): A string representation showing who performed what action and when.
        """
        return f"Action by {self.simulated_user_session.real_user} as {self.simulated_user_session.simulated_user} on" \
               f" {self.timestamp}"



from setuptools import setup, find_packages

setup(
    name='django-mimicry',
    version='1.7',
    packages=find_packages(exclude=['djangoMimicryProject']),
    include_package_data=True,
    license='MIT License',
    description='A Django app to simulate user sessions.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/YidiSprei/djangoMimicry.git',
    author='Yidi Sprei',
    author_email='yidi.sprei@infuzu.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



from django.apps import AppConfig


class MimicryConfig(AppConfig):
    """
    Configuration class for the Mimicry application.

    Attributes:
    - default_auto_field: Specifies the type of auto-created primary key.
    - name: The full Python path of the application.
    - verbose_name: Human-readable name for the application.

    Methods:
    - ready: This method is called when the application is ready. It's used here to import signals.
    """
    default_auto_field: str = 'django.db.models.BigAutoField'
    name: str = 'mimicry'
    verbose_name: str = "Mimicry"

    # noinspection PyUnresolvedReferences
    def ready(self) -> None:
        """
        Import signals when the application is ready.
        This ensures that the signal handlers are connected when the app is loaded.
        """
        from . import signals
        from . import checks



"""
URL Configuration for the simulate_user app.

This module contains the URL patterns for the app's views.

Routes:
- `/switch_user/`: The endpoint to handle the switching or simulating of different users.

Functions:
- `switch_user`: View function to handle user switching logic.

Usage:
Make sure to include these URL patterns in the project's main `urls.py` file using Django's `include()` function.
"""

from django.urls import path
from .views import switch_user

urlpatterns: list = [
    path('switch_user/', switch_user, name='mimicry_switch_user'),
]
